/*
 * @Author       : ChenJie
 * @Date         : 2022-01-23 13:45:09
 * @Version      : V3.0
 * @LastEditors  : ChenJie
 * @LastEditTime : 2022-01-23 13:59:52
 * @Description  : file content
 * @FilePath     : \PLAT\project\ec616_0h00\apps\qx_app\src\AT_CMD\AppFuncLib.h
 */
/*
 * AppFunc.h
 *
 *  Created on: 2022Äê1ÔÂ21ÈÕ
 *      Author: QiXiang_CHENJIE
 */

#ifndef APPFUNCLIB_H_
#define APPFUNCLIB_H_
#include "hal_adapter.h"
uint16 ATstrdel(char *str);
uint16 mstrlen(const char *s);
int mstrncmp(const char *s1, const char *s2, int n);
unsigned char HexToChar(unsigned char bHex);
unsigned char CharToHex(unsigned char bChar);
uint8 AtStrCompare(const char *a, const char *b);
unsigned short CRC16_Modbus ( unsigned char *pdata, int len);
#endif /* APPFUNCLIB_H_ */
